@@ -7,7 +7,7 @@ from typing import Optional, Union, Tuple, List, Set, Dict
 import nonebot
 from nonebot.log import logger
 from pydantic import BaseModel, parse_obj_as
-from ruamel.yaml import YAML, YAMLError
+from ruamel.yaml import YAML, YAMLError, CommentedMap, CommentedSeq
 
 from .config import c
 
@@ -49,10 +49,16 @@ def get_namespace(namespace: str, required: bool, path_override: Path = None) ->
 
 
 @nonebot_driver.on_startup
-def reload():
+def reload(force: bool = False) -> bool:
     """
     使所有权限组在下一次使用时重新从配置加载。
+
+    :param force: 强制重新加载，忽略未保存的修改。
+    :return: 因有未保存的修改而没有重新加载时返回 False ，否则返回 True 。
     """
+    if not force and any(x.dirty for x in loaded.values()):
+        return False
+
     loaded.clear()
     loaded_by_path.clear()
     plugin_namespaces.clear()
@@ -72,16 +78,36 @@ def reload():
             namespace = get_namespace(name, True, handler.preset_)
             plugin_namespaces.append(namespace)
 
+    # 生成全局组默认配置
+    if not global_.path.is_file():
+        global_.dirty = True
+        global_.save()
+    for name in ['group', 'user']:
+        namespace = get_namespace(name, False)
+        if not namespace.path.is_file():
+            namespace.add_group(42, 'Example')
+            namespace.save()
+
+    return True
+
 
 @nonebot_driver.on_shutdown
-def save_all():
+def save_all() -> bool:
+    """
+    保存所有权限配置。
+
+    :return: 是否全部保存成功。
+    """
     logger.debug('Saving permissions')
+    failed = False
     for k, v in loaded.items():
         try:
             v.save()
         except Exception as e:
             _ = e
+            failed = True
             logger.exception('Failed to save namespace {}', k)
+    return not failed
 
 
 try:
@@ -117,7 +143,7 @@ class Namespace:
             self.config = {}
             self.modifiable = False
         elif not required and not path.is_file():
-            self.config = {}
+            self.config = CommentedMap()
         else:
             try:
                 doc = yaml.load(path)
@@ -125,17 +151,18 @@ class Namespace:
                 logger.exception('Failed to load namespace {} ({})', namespace, path)
                 doc = {}
 
-            if not isinstance(doc, dict):
+            if not isinstance(doc, CommentedMap):
                 logger.error('Expect a dict: {} ({})', namespace, path)
                 doc = {}
 
-            self.config: Dict[Union[str, int], dict] = doc
+            self.config: CommentedMap[Union[str, int], dict] = doc
 
     def save(self):
         """
         把本名称空间保存到硬盘上。若没有修改过则不做任何事。
         """
         if self.modifiable and self.dirty:
+            self.path.parent.mkdir(parents=True, exist_ok=True)
             yaml.dump(self.config, self.path)
             self.dirty = False
 
@@ -204,18 +231,21 @@ class Namespace:
         yield self.config[name] if name is not None else None
         self.dirty = True
 
-    def add_group(self, name: Union[str, int]):
+    def add_group(self, name: Union[str, int], comment: str = None):
         """
         创建权限组。
 
         :param name: 权限组名。
+        :param comment: 注释。
         :raise KeyError: 权限组已存在。
         :raise TypeError: 名称空间不可修改。
         """
         with self.modifying():
             if name in self.config:
                 raise KeyError('Duplicate group')
-            self.config[name] = {}
+            self.config[name] = CommentedMap(permissions=CommentedSeq())
+            if comment is not None:
+                self.config.yaml_add_eol_comment(comment, name)
             self.groups.pop(name, None)
 
     def remove_group(self, name: Union[str, int], force: bool):
@@ -317,11 +347,12 @@ class PermissionGroup:
 
         del self.referer
 
-    def add(self, item: str):
+    def add(self, item: str, comment: str = None):
         """
         添加权限描述。
 
         :param item: 权限描述。
+        :param comment: 注释。
         :raise ValueError: 权限组中已有指定描述。
         :raise TypeError: 权限组不可修改。
         """
@@ -336,8 +367,10 @@ class PermissionGroup:
                 raise ValueError('Duplicate item')
             target.add(perm)
             self.cache.clear()
-            permissions = desc.setdefault('permissions', [])
+            permissions: CommentedSeq = desc.setdefault('permissions', CommentedSeq())
             permissions.append(item)
+            if comment is not None:
+                permissions.yaml_add_eol_comment(comment, len(permissions) - 1)  # yaml_add_eol_comment 不支持负数下标
 
     def remove(self, item: str):
         """
@@ -411,3 +444,9 @@ class NullPermissionGroup(PermissionGroup):
 
     def populate(self, desc, referer):
         raise TypeError
+
+    def add(self, item: str, comment: str = None):
+        raise TypeError
+
+    def remove(self, item: str):
+        raise TypeError
